#include "widget.h"
#include "ui_widget.h"

#ifdef __cplusplus
extern "C" {
#endif

#include <libacpi.h>

#ifdef __cplusplus
}
#endif

#include <QFileDialog>

Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget),
    m_adapterWidget(nullptr)
{
    ui->setupUi(this);
}

Widget::~Widget()
{
    delete ui;
}

void Widget::on_pushButton_clicked()
{
    // Выделяем память под глобальную структуру данных бибилотеки
    global_t *global = static_cast<global_t*>(malloc(sizeof(global_t)));

    // Инициализируем структуру для
    init_acpi_batt(global);     // батареи
    init_acpi_acadapt(global);  // адаптера питания
    init_acpi_fan(global);      // вентиляторов
    init_acpi_thermal(global);  // тепловых зон

    // Battery
    battery_t *binfo;

    // Если нашлись батареи, то считываем в цикле данные и добавляем виджеты
    for(int i=0; i < global->batt_count; i++)
    {
        read_acpi_batt(i);      // считываем данные
        // которые считываются в специальный массив batteries - он есть в самой библиотеке
        binfo = &batteries[i];

        BatteryWidget* butteryWidget;
        // Создаём виджет
        if (m_butteryWidgetList.count() <= i)
        {
            butteryWidget = new BatteryWidget(this);
            butteryWidget->setTitle(QString(binfo->name));
            m_butteryWidgetList.append(butteryWidget);
            ui->verticalLayout->addWidget(butteryWidget);
        }
        else
        {
            // Или забираем его из списка виджетов
            butteryWidget = m_butteryWidgetList.at(i);
        }

        // Устанавливаем
        butteryWidget->setPercentage(binfo->percentage);        // Процент заряда
        butteryWidget->setChargeTime(binfo->charge_time);       // Время до полного заряда
        butteryWidget->setRemainingTime(binfo->remaining_time); // Время до полного разряда

        // Определяем статус заряда
        switch (binfo->charge_state)
        {
        case C_CHARGE:
            butteryWidget->setChargeState("Charging");
            break;
        case C_DISCHARGE:
            butteryWidget->setChargeState("Discharging");
        case C_CHARGED:
            butteryWidget->setChargeState("Charged");
        case C_NOINFO:
            butteryWidget->setChargeState("No info");
        default:
            butteryWidget->setChargeState("Error");
            break;
        }
    }

    // Adapter
    // Настройка виджета для адптера питания
    adapter_t *ac = &global->adapt;

    if (!m_adapterWidget)
    {
        m_adapterWidget = new AdapterWidget(this);
        ui->verticalLayout->addWidget(m_adapterWidget);
    }

    // Здесь проверяем только, подключён или нет
    switch (ac->ac_state)
    {
    case P_AC:
        m_adapterWidget->setStatus("Runs on AC");
        break;
    case P_BATT:
        m_adapterWidget->setStatus("Runs on battery");
    default:
        m_adapterWidget->setStatus("Error");
        break;
    }

    // Fan
    fan_t *finfo;

    // Если нашлись вентиляторы, то считываем в цикле данные и добавляем виджеты
    for(int i=0; i < global->fan_count; i++)
    {
        read_acpi_fan(i);   // считываем данные
        // которые считываются в специальный массив fans - он есть в самой библиотеке
        finfo = &fans[i];
        FanWidget* fanWidget;

        // Создаём виджет
        if (m_fanWidgetList.count() <= i)
        {
            fanWidget = new FanWidget(this);
            fanWidget->setTitle(QString(finfo->name));
            m_fanWidgetList.append(fanWidget);
            ui->verticalLayout->addWidget(fanWidget);
        }
        else
        {
            // Или забираем его из списка виджетов
            fanWidget = m_fanWidgetList.at(i);
        }

        // Устанавливаем статус, включён или нет
        switch (finfo->fan_state)
        {
        case F_ON:
            fanWidget->setStatus("ON");
            break;
        case F_OFF:
            fanWidget->setStatus("OFF");
        default:
            fanWidget->setStatus("Error");
            break;
        }
    }

    // Thermal
    thermal_t *tinfo;

    // Если нашлись тепловые зоны, то считываем в цикле данные и добавляем виджеты
    for(int i=0; i < global->fan_count; i++)
    {
        read_acpi_zone(i, global);  // считываем данные
        // которые считываются в специальный массив thermals - он есть в самой библиотеке
        tinfo = &thermals[i];
        ThermalWidget* thermalWidget;

        // Создаём виджет
        if (m_thermalWidgetList.count() <= i)
        {
            thermalWidget = new ThermalWidget(this);
            thermalWidget->setTitle(QString(tinfo->name));
            m_thermalWidgetList.append(thermalWidget);
            ui->verticalLayout->addWidget(thermalWidget);
        }
        else
        {
            // Или забираем его из списка виджетов
            thermalWidget = m_thermalWidgetList.at(i);
        }

        thermalWidget->setTemperature(tinfo->temperature);  // Устанавливаем температуру
        thermalWidget->setFrequency(tinfo->frequency);      // и частоту
    }

    free(global);
}
